# -*- coding: utf-8 -*-

# stdlib imports
import os
import sys
import re

# third-party imports
import click

# module imports
from .helper import check_file, return_filehandle, create_directories, setup_logging


def prefix_fasta(target, gnm, genus, species, infra_id, key, logger):
    """Prefix FASTA files for data store standard"""
    re_header = re.compile(r"^>(\S+)\s*(.*)")  # match header
    prefix = "{}{}".format(genus[:3].lower(), species[:2].lower())  # gensp
    new_file_raw = "{}.{}.{}.{}.genome_main.fna".format(prefix, infra_id, gnm, key)
    species_dir = "./{}_{}".format(genus.capitalize(), species)
    new_file_dir = "{}/{}.{}.{}".format(species_dir, infra_id, gnm, key)  # setup for ds
    create_directories(new_file_dir)  # make genus species dir for output
    fasta_file_path = os.path.abspath("{}/{}".format(new_file_dir, new_file_raw))
    new_fasta = open(fasta_file_path, "w")
    name_prefix = "{}.{}.{}".format(prefix, infra_id, gnm)
    fh = return_filehandle(target)
    with fh as gopen:
        for line in gopen:
            line = line.rstrip()
            if re_header.match(line):  # header line
                parsed_header = re_header.search(line).groups()
                logger.debug(line)
                logger.debug(parsed_header)
                hid = ""
                desc = ""
                new_header = ""
                if isinstance(parsed_header, str):  # check tuple
                    hid = parsed_header
                    new_header = ">{}.{}".format(name_prefix, hid)
                    logger.debug(hid)
                else:
                    if len(parsed_header) == 2:  # header and description
                        hid = parsed_header[0]  # header
                        desc = parsed_header[1]  # description
                        new_header = ">{}.{} {}".format(name_prefix, hid, desc)
                        logger.debug(hid)
                        logger.debug(desc)
                        logger.debug(new_header)
                    else:
                        logger.error("header {} looks odd...".format(line))
                        sys.exit(1)
                new_fasta.write(new_header + "\n")  # write new header
            else:  # sequence lines
                new_fasta.write(line + "\n")
    new_fasta.close()
    if not check_file(fasta_file_path):
        logger.error(
            "Output file {} not found for normalize fasta".format(fasta_file_path)
        )
        sys.exit(1)  # new file not found return False
    return fasta_file_path


def type_rank(hierarchy, feature_type):
    if feature_type in hierarchy:
        return hierarchy[feature_type]["rank"]
    else:
        return 1000  # no feature id, no children, no order


def update_hierarchy(hierarchy, feature_type, parent_types):
    """Breaks input gff3 line into attributes.

       Determines feature type hierarchy using type and attributes fields
    """
    if not parent_types:  # this feature must be a root
        if feature_type not in hierarchy:  # add to hierarchy
            hierarchy[feature_type] = {"children": [], "parents": [], "rank": 0}
        hierarchy[feature_type]["rank"] = 1  # rank is 1 because no parents
    else:  # feature has parents
        if feature_type not in hierarchy:
            hierarchy[feature_type] = {"children": [], "parents": [], "rank": 0}
        hierarchy[feature_type]["parents"] = parent_types
        for p in parent_types:  # set children and aprents for all p
            if hierarchy.get(p):
                if feature_type not in hierarchy[p]["children"]:
                    hierarchy[p]["children"].append(feature_type)
            else:
                hierarchy[p] = {"children": [feature_type], "parents": [], "rank": 0}


def prefix_gff3(
    target, gnm, ann, genus, species, infra_id, unique_key, logger, sort_only
):
    """Prefixes GFF3 file for datastore standard.  Sorts for Tabix Indexing"""
    prefix = "{}{}".format(genus[:3].lower(), species[:2].lower())  # gensp
    key = unique_key
    target = os.path.abspath(target)
    new_file_raw = "{}.{}.{}.{}.{}.gene_models_main.gff3".format(
        prefix, infra_id, gnm, ann, key
    )
    species_dir = "./{}_{}".format(genus.capitalize(), species)
    new_file_dir = "{}/{}.{}.{}.{}".format(
        species_dir, infra_id, gnm, ann, key
    )  # setup for ds
    create_directories(new_file_dir)  # make genus species dir for output
    gff_file_path = os.path.abspath("{}/{}".format(new_file_dir, new_file_raw))
    new_gff = open(gff_file_path, "w")
    get_id = re.compile("ID=([^;]+)")  # gff3 get id string
    get_name = re.compile("Name=([^;]+)")  # get name string
    get_parents = re.compile("Parent=([^;]+)")  # get parents
    gff3_lines = []  # will be sorted after loading and hierarchy
    sub_tree = {}  # feature hierarchy sub tree
    type_hierarchy = {}  # type hierarchy for features, will be ranked
    prefix_name = False
    fh = return_filehandle(target)  # get filehandle for target
    with fh as gopen:
        for line in gopen:
            line = line.rstrip()
            if not line:
                continue
            if line.startswith("#"):  # header
                if sort_only:  # do not prefix
                    new_gff.write("{}\n".format(line))
                    continue
                if line.startswith("##sequence-region"):  # replace header
                    fields = re.split(r"\s+", line)
                    ref_name = "{}.{}.{}.{}".format(prefix, infra_id, gnm, fields[1])
                    line = re.sub(fields[1], ref_name, line)
                new_gff.write("{}\n".format(line))
                continue
            fields = line.split("\t")
            gff3_lines.append(fields)
            feature_id = get_id.search(fields[-1])
            parent_ids = get_parents.search(fields[-1])
            if parent_ids:
                parent_ids = parent_ids.group(1).split(",")
            else:
                parent_ids = []
            if not feature_id:
                continue
            feature_id = feature_id.group(1)
            sub_tree[feature_id] = {
                "type": fields[2],  # parent child
                "parent_ids": parent_ids,
            }
    for f in sub_tree:
        feature_type = sub_tree[f]["type"]
        parent_types = []
        for p in sub_tree[f]["parent_ids"]:
            parent_type = sub_tree[p]["type"]
            if not parent_type:  # must have type
                logger.error("could not find type for parent {}".format(parent_type))
                sys.exit(1)  # error no type
            parent_types.append(parent_type)
        update_hierarchy(type_hierarchy, feature_type, parent_types)
    ranking = 1  # switch to stop ranking in while below
    while ranking:  # this is over-engineered for tabix indexing, but tis cool
        check = 0  # switch to indicate no features unranked
        for t in sorted(
            type_hierarchy, key=lambda k: type_hierarchy[k]["rank"], reverse=True
        ):
            if type_hierarchy[t]["rank"]:  # rank !=0
                for c in type_hierarchy[t]["children"]:
                    if not type_hierarchy[c]["rank"]:
                        type_hierarchy[c]["rank"] = type_hierarchy[t]["rank"] + 1
            else:  # rank == 0
                check = 1
        if not check:  # escape loop when all features are ranked
            ranking = 0
    feature_prefix = "{}.{}.{}.{}".format(prefix, infra_id, gnm, ann)
    for l in sorted(
        gff3_lines, key=lambda x: (x[0], int(x[3]), type_rank(type_hierarchy, x[2]))
    ):  # rank by chromosome, start, type_rank and stop
        if sort_only:  # do not prefix or change IDs
            l = "\t".join(l)
            new_gff.write("{}\n".format(l))
            continue
        l[0] = "{}.{}.{}.{}".format(prefix, infra_id, gnm, l[0])  # rename
        l = "\t".join(l)
        feature_id = get_id.search(l)
        feature_name = get_name.search(l)
        feature_parents = get_parents.search(l)
        if feature_id:  # if id set new id
            new_id = "{}.{}".format(feature_prefix, feature_id.group(1))
            l = get_id.sub("ID={}".format(new_id), l)
        if feature_name and prefix_name:  # if name and flag set new name
            new_name = "{}.{}".format(feature_prefix, feature_name.group(1))
            l = get_name.sub("Name={}".format(new_name), l)
        if feature_parents:  # parents set new parent ids
            parent_ids = feature_parents.group(1).split(",")
            new_ids = []
            for p in parent_ids:  # for all parents
                new_id = "{}.{}".format(feature_prefix, p)
                new_ids.append(new_id)
            new_ids = ",".join(new_ids)  # set delimiter for multiple parents
            l = get_parents.sub("Parent={}".format(new_ids), l)
        new_gff.write("{}\n".format(l))
    new_gff.close()
    if not check_file(gff_file_path):  # check for output error if not found
        logger.error("Output file {} not found for normalize gff".format(gff_file_path))
        sys.exit(1)  # new file not found return False
    return gff_file_path  # return path to new gff file


@click.command()
@click.option("--target", help="""TARGETS can be files or directories or both""")
@click.option("--gnm", help="""Genome Version for Normalizer.""")
@click.option("--ann", help="""Annotation Version for Normalizer.""")
@click.option("--genus", metavar="<STRING>", help="""Genus of organism input file""")
@click.option(
    "--species", metavar="<STRING>", help="""Species of organism input file"""
)
@click.option(
    "--infra_id",
    metavar="<STRING>",
    help="""Line or infraspecific identifier.  ex. A17_HM341""",
)
@click.option(
    "--unique_key",
    metavar="<STRING, len=4>",
    help="""4 Character unique idenfier (get from spreadsheet)""",
)
@click.option(
    "--sort_only",
    is_flag=True,
    help="""Performing sorting only (only applies to gff3 files)""",
)
@click.option(
    "--log_file",
    metavar="<FILE>",
    default="./normalizer_prefix.log",
    help="""File to write log to. (default:./normalizer_prefix.log)""",
)
@click.option(
    "--log_level",
    metavar="<LOGLEVEL>",
    default="INFO",
    help="""Log level: DEBUG, INFO, WARNING, ERROR, CRITICAL (default:INFO)""",
)
def cli(
    target,
    gnm,
    ann,
    genus,
    species,
    infra_id,
    unique_key,
    sort_only,
    log_file,
    log_level,
):
    """Prefixing for Datastore standard.

       full prefix = gensp.infra_id.gnm.[ann]
       file prefix = gensp.infra_id.gnm.[ann].unique_key

       If the File is a GFF3 it will be sorted.

       A directory will be created in ./Genus_species containing corretly
       named subdirectories and files.

       You can run all of the other steps on the results of this command.
    """
    file_types = ["fna", "fasta", "fa", "gff", "gff3", "ADD MORE"]
    fasta = ["fna", "fasta", "fa"]
    gff3 = ["gff", "gff3"]
    logger = setup_logging(log_file, log_level)
    if not (target and gnm and genus and species and infra_id and unique_key):
        logger.error(
            "--target, --species, --genus, --gnm, --infra_id, and --unique_key are required"
        )
        sys.exit(1)
    target = os.path.abspath(target)
    gnm = "gnm{}".format(gnm)  # format user provided argument for gnm
    ann = "ann{}".format(ann)  # format user provided argument for ann
    file_attributes = target.split(".")
    new_file = False
    if len(file_attributes) < 2:  # file must have an extension
        error_message = """Target {} does not have a type or attributes.  File must end in either gz, bgz, fasta, fna, fa, gff, or gff3.""".format(
            target
        )
        logger.error(error_message)
        sys.exit(1)
    file_type = file_attributes[-1]
    if file_type == "gz" or file_type == "bgz":  # if compressed get upstream
        file_type = file_attributes[-2]
    if file_type not in file_types:  # must be a type in file_types
        logger.error("File {} is not a type in {}".format(target, file_types))
        sys.exit(1)
    if file_type in fasta:
        logger.info("Target is a FASTA file prefixing...")
        new_file = prefix_fasta(
            target, gnm, genus, species, infra_id, unique_key, logger
        )  # returns new file path
        logger.info("Prefixing done, final file: {}".format(new_file))
    if file_type in gff3:
        if sort_only:
            logger.info("Target is a gff3 file sorting ONLY...")
        else:
            logger.info("Target is a gff3 file prefixing and sorting...")
        new_file = prefix_gff3(
            target, gnm, ann, genus, species, infra_id, unique_key, logger, sort_only
        )  # returns new file path
        logger.info("Prefixing done, final file: {}".format(new_file))
    if not new_file:  # no new file was generated, must have errored
        logger.error("Normalizer FAILED.  See Log.")
        sys.exit(1)
    return new_file



#!/usr/bin/env python

import os
import sys
import gzip
import click
import logging
from . import Detector


@click.command()
@click.option(
    "--target", multiple=True, help="""TARGETS can be files or directories or both"""
)
@click.option("--no_busco", is_flag=True, help="""Disable BUSCO""")
@click.option(
    "--no_nodes", is_flag=True, help="""Disable DSCensor Stats and Node Generation"""
)
@click.option(
    "--log_file",
    metavar="<FILE>",
    default="./bionorm.log",
    help="""File to write log to. (default:./bionorm.log)""",
)
@click.option(
    "--log_level",
    metavar="<LOGLEVEL>",
    default="INFO",
    help="""Log level: DEBUG, INFO, WARNING, ERROR, CRITICAL (default:INFO)""",
)
# @click.option('--normalize', is_flag=True,
#             help='''Normalizes provided files.
# Incongruencies in FASTA will be corrected if the provided genome name
# passes checks.
# The gff file will be tidied if it fails gff3 validation in gt:
#    gt gff3 -sort -tidy -retainids input.gff3 > out.gff3
# ''')
def cli(target, no_busco, no_nodes, log_file, log_level):
    """incongruency_detector:

        Detect Incongruencies with LIS Data Store Standard
    """
    if not target:
        print("Must specify at least one --target.  Run with --help for usage")
        sys.exit(1)
    data_store_home = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    templates_dir = "{}/templates".format(data_store_home)
    readme_template = "{}/template__README.KEY.yml".format(templates_dir)
    options = {
        "log_level": log_level,
        "log_file": log_file,
        "no_busco": no_busco,
        "no_nodes": no_nodes,
        "readme_template": readme_template,
    }
    for t in target:
        detector = Detector(t, **options)  # initializers
        detector.detect_incongruencies()  # class method runs all detection methods


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
"""Implements commands related to configuration parameters
"""
# standard library imports
import sys

# third-party imports
import click

# module imports
from . import cli
from .common import *

# private context function
_ctx = click.get_current_context


@cli.command()
def show_config():
    """Print location and contents of config file.

        Example:
            bionorm -v show_config
    """
    global config_obj
    if config_obj.config_dict == {}:
        logger.info("No configuration file was found.")
    else:
        logger.info('Configuration file path is "%s".', config_obj.path)
        for key in config_obj.config_dict.keys():
            logger.info("  %s: %s", key, config_obj.config_dict[key])


@cli.command()
@click.argument("plot_type", type=str, nargs=-1)
def set_plot_type(plot_type):
    """Define label associated with a set.
    """
    global config_obj
    plot_types = ["svg", "jpg", "ps", "pdf", "svgr", "png", "tiff", "eps"]
    if len(plot_type) == 0:
        logger.info("Supported plot types are:")
        for typename in plot_types:
            logger.info("   %s", typename)
    elif len(plot_type) > 1:
        logger.error("Only one argument for set_plot_type is allowed.")
        sys.exit(1)
    elif plot_type[0] not in plot_types:
        logger.error('Plot type "%s" is not defined.', plot_type[0])
        logger.info("Supported plot types are:")
        for typename in plot_types:
            logger.info("   %s", typename)
        sys.exit(1)
    else:
        config_obj.config_dict["plot_type"] = plot_type[0]
        logger.info("Plot type is now %s.", plot_type[0])
        config_obj.write_config_dict()


@cli.command()
@click.argument("dir", type=str, default="")
def init_config_file(dir):
    """Initialize a configuration file.

    :param dir: Optional directory in which to initialize the file.
    If not present, the system-dependent default application directory
    will be used.  If this argument is '.', then the current working
    directory will be used.  This argument accepts tilde expansions.
    """
    global config_obj
    config_obj.write_config_dict(dir=dir, config_dict={})



# -*- coding: utf-8 -*-
"""bionorm -- normalize and validate genomic data files.

This tool is used to ingest data into the Legume Federation Data Store
(https://www.legumefederation.org/en/data-store/) and is useful anytime
genomes must be formatted and checked for consistency.
"""

# standard library imports
import warnings
from pkg_resources import iter_entry_points

# third-party imports
import click
from pkg_resources import iter_entry_points
from click_plugins import with_plugins

# module defs
from .common import logger, PROGRAM_NAME
from .cli import Logging_CLI_Builder

# global constants
DEFAULT_FIRST_N = 0


# define parser options
parser_options = [
    {
        "args": ("--progress",),
        "kwargs": {
            "is_flag": True,
            "show_default": True,
            "default": False,
            "help": "Show a progress bar.",
        },
    },
    {
        "args": ("--first_n",),
        "kwargs": {
            "default": 0,
            help: "Process only this many records. [default: all]",
        },
    },
    {
        "args": ("--warnings_as_errors", "-e"),
        "kwargs": {
            "is_flag": True,
            "show_default": True,
            "default": False,
            "help": "Warnings cause exceptions.",
        },
    },
]
# define custom CLI check function, parameters must be keyworded
def bionorm_check(warnings_as_errors=False, **others):
    if warnings_as_errors:
        logger.warn("Runtime warnings (e.g., from pandas) will cause exceptions")
        warnings.filterwarnings("error")


# define the CLI
cli_builder = Logging_CLI_Builder(
    PROGRAM_NAME, logger, global_options_list=parser_options
)

# create CLI
@with_plugins(iter_entry_points(PROGRAM_NAME + ".cli_plugins"))
@click.group(
    epilog=cli_builder.author
    + " <"
    + cli_builder.email
    + ">.  "
    + cli_builder.copyright
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    show_default=True,
    default=False,
    help="Log debugging info to stderr.",
)
@click.option(
    "-q",
    "--quiet",
    is_flag=True,
    show_default=True,
    default=False,
    help="Suppress logging to stderr.",
)
@click.option(
    "--log/--no_log", is_flag=True, show_default=True, default=True, help="Log to file."
)
@click.option(
    "--progress",
    is_flag=True,
    show_default=True,
    default=False,
    help="Show a progress bar.",
)
@click.option(
    "--first_n",
    default=DEFAULT_FIRST_N,
    help="Process only this many records. [default: all]",
)
@click.option(
    "--warnings_as_errors",
    "-e",
    is_flag=True,
    show_default=True,
    default=False,
    help="Warnings cause exceptions.",
)
@click.version_option(version=cli_builder.version, prog_name=PROGRAM_NAME)
@cli_builder.init_dual_logger()
@cli_builder.init_user_context_obj(extra_args=["progress", "first_n"])
def cli(verbose, quiet, log, **kwargs):
    """bionorm -- normalize and verify genomic data files

        If COMMAND is present, and --no_log was not invoked,
        a log file named bionorm-COMMAND.log
        will be written in the ./logs/ directory.
    """
    bionorm_check(**kwargs)


cli_builder.set_cli_func(cli)

# Define some cli-related commands
test_log = cli_builder.test_log_func()
show_context = cli_builder.show_context_func()

# import other functions
from .config import show_config, init_config_file



#!/usr/bin/env python

from .helper import check_file, return_filehandle
import os
import sys
import logging
import re
import subprocess
import hashlib
import requests
import json
from glob import glob
from os import path

sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))


class genome_main:
    def __init__(self, detector, **kwargs):
        self.detector = detector
        self.target = detector.target
        self.logger = detector.logger

    def run(self):
        """Runs checks"""
        logger = self.logger
        target = self.target
        logger.info("Performing Naming Checks for {}\n".format(target))
        if not self.check_genome_main():
            logger.error("Naming Check FAILED")
            return False
        logger.info("Naming Looks Correct\n")
        logger.info("Checking Genome FASTA\n")
        if not self.check_genome_fasta():
            logger.error("FASTA Check FAILED")
            return False
        logger.info("ALL GENOME CHECKS PASSED\n")
        return True

    def check_genome_main(self):
        """accepts a list of genome attributes split by "."

           https://github.com/LegumeFederation/datastore/issues/23

           checks these file attributes to ensure they are correct
        """
        target = self.target
        logger = self.logger
        attr = os.path.basename(target).split(".")  # split on delimiter
        if len(attr) != 7:  # should be 7 fields
            logger.error("File did not have 7 fields! {}".format(attr))
            return False
        if len(attr[0]) != 5:  # should be 5 letter prefix
            logger.error("File must have 5 letter prefix, not {}".format(attr[0]))
            return False
        if not attr[2].startswith("gnm"):  # should be gnm type
            logger.error("File should have gnm in field 3, not {}".format(attr[2]))
            return False
        gnm_v = attr[2].replace("gnm", "")
        try:
            int(gnm_v)
        except ValueError:  # best way to check for int in python2
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if len(gnm_v.split(".")) > 1:  # check for float
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if not attr[5] == "fna":  # should be fna type
            logger.error("File should be fna not {}".format(attr[5]))
            return False
        if not attr[6] == "gz":  # should be gzip compressed
            logger.error("Last field should be gz, not {}".format(attr[6]))
            return False
        logger.info("Genome Naming Looks Correct")
        return True

    def check_genome_fasta(self):
        """Confirms that headers in fasta genome_main conform with standard

           PUT SOME RULE REFERENCE HERE
        """
        logger = self.logger
        fasta = self.target  # get fasta file
        attr = os.path.basename(fasta).split(".")  # get attributes for naming
        true_header = ".".join(attr[:3])
        fh = return_filehandle(fasta)  # get file handle, text/gz
        # grab header and description
        re_header = re.compile(r"^>(\S+)\s*(.*)")
        passed = True
        with fh as gopen:
            for line in gopen:
                line = line.rstrip()
                if not line:
                    continue
                if re_header.match(line):  # check for fasta header
                    hid = re_header.search(line)
                    if hid:
                        logger.debug(hid.groups(0))
                        if isinstance(hid, str):  # check for tuple
                            hid = hid.groups(0)
                        else:
                            hid = hid.groups(0)[0]  # get id portion of header
                    else:
                        logger.error("Header {} looks odd...".format(line))
                        return False
                    logger.debug(hid)
                    self.detector.fasta_ids[hid] = 1
                    standard_header = true_header + "." + hid
                    if not hid.startswith(true_header):
                        logger.warning(
                            (
                                "Inconsistency {} ".format(hid)
                                + "Should be {}".format(standard_header)
                            )
                        )
                        passed = False
        return passed


class gene_models_main:
    def __init__(self, detector, **kwargs):
        self.detector = detector
        self.target = detector.target
        self.logger = detector.logger
        self.fasta_ids = detector.fasta_ids

    def run(self):
        """Run checks"""
        logger = self.logger
        target = self.target
        logger.info("Checking Gene Models File Naming...")
        if not self.check_gene_models_main():
            logger.error("Gene Model File Naming FAILED")
            return False
        logger.info("Naming Looks Correct\n")
        logger.info("Validating GFF3 with gt...")
        if not self.gt_gff3_validate():
            logger.error("gt GFF3 Validation FAILED")
            return False
        logger.info("GFF3 is Valid\n")
        logger.info("Checking Congruency Between Genome and Gene Models...")
        if not self.check_seqid_attributes():
            logger.error("Genome and Gene Models are not Congruent FAILED")
            return False
        logger.info("ALL GENE MODELS CHECKS PASSED\n")
        return True

    def check_gene_models_main(self):
        """accepts a list of annotation attributes split by "."

           https://github.com/LegumeFederation/datastore/issues/23

           checks these file attributes to ensure they are correct
        """
        target = self.target
        logger = self.logger
        attr = os.path.basename(target).split(".")  # split on delimiter
        if len(attr) != 8:  # should be 8 fields
            logger.error("File did not have 7 fields! {}".format(attr))
            return False
        if len(attr[0]) != 5:  # should be 5 letter prefix
            logger.error("File must have 5 letter prefix, not {}".format(attr[0]))
            return False
        if not attr[2].startswith("gnm"):  # should be gnm type
            logger.error("File should have gnm in field 3, not {}".format(attr[2]))
            return False
        gnm_v = attr[2].replace("gnm", "")
        try:
            int(gnm_v)
        except ValueError:  # best way to check for int in python2
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if len(gnm_v.split(".")) > 1:  # check for float
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if not attr[3].startswith("ann"):  # should be gnm type
            logger.error("File should have ann in field 4, not {}".format(attr[2]))
            return False
        ann_v = attr[3].replace("ann", "")
        try:
            int(ann_v)
        except ValueError:  # best way to check for int in python2
            logger.error("ann version must be integer not {}".format(ann_v))
            return False
        if len(ann_v.split(".")) > 1:  # check for float
            logger.error("ann version must be integer not {}".format(gnm_v))
            return False
        if not attr[6] == "gff3":  # should be gff3 type
            logger.error("File should be gff3 not {}".format(attr[6]))
            return False
        if not attr[7] == "gz":  # should be gzip compressed
            logger.error("Last field should be gz, not {}".format(attr[6]))
            return False
        return True

    def gt_gff3_validate(self):
        """Confirms that gff3 files pass gt validation

           https://github.com/LegumeFederation/datastore/issues/23
        """
        gff = self.target
        logger = self.logger
        gff_name = os.path.basename(gff)
        gt_report = "./{}_gt_gff3validator_report.txt".format(gff_name)
        gt_cmd = "(gt gff3validator {} 2>&1) > {}".format(gff, gt_report)
        logger.debug(gt_cmd)
        exit_val = subprocess.call(gt_cmd, shell=True)  # get gt exit_val
        logger.debug(exit_val)
        if exit_val:
            return False
        return True

    def check_seqid_attributes(self):
        """Confirms that gff3 seqid exists in genome_main if provided

           checks ID and Name from gff3 attributes field

           https://github.com/LegumeFederation/datastore/issues/23
        """
        gff = self.target
        logger = self.logger
        fasta_ids = self.fasta_ids  # list of FASTA IDS from Reference
        logger.debug(fasta_ids)
        fh = return_filehandle(gff)
        file_name = os.path.basename(gff)
        # ID should start with this string
        true_id = ".".join(file_name.split(".")[:4])
        true_name = file_name.split(".")[0]  # maybe this should include infra
        #        get_id_name = re.compile("^ID=(.+?);.*Name=(.+?);")
        get_id = re.compile("ID=([^;]+)")
        lines = 0
        seen = {}
        passed = True
        with fh as gopen:
            for line in gopen:
                line = line.rstrip()
                lines += 1
                if not line or line.startswith("#"):
                    continue
                columns = line.split("\t")  # get gff3 fields
                seqid = columns[0]  # seqid according to the spec
                seqid = seqid.rstrip()
                logger.debug(line)
                logger.debug(seqid)
                if self.fasta_ids:  # if genome_main make sure seqids exist
                    if seqid not in fasta_ids:  # fasta header check
                        logger.debug(seqid)
                        if seqid not in seen:
                            logger.error("{} not found in genome_main".format(seqid))
                            seen[seqid] = 1
                        passed = False
                feature_type = columns[2]  # get type
                attributes = columns[8]  # attributes ';' delimited
                logger.debug(feature_type)
                if feature_type != "gene":  # only check genes (for now)
                    continue
                #                if not get_id_name.match(attributes):  # check for ID and Name
                if not get_id.search(attributes):  # check for ID and Name
                    logger.error("No ID and Name attributes. line {}".format(lines))
                    passed = False
                else:
                    #                    groups = get_id_name.search(attributes).groups()
                    feature_id = get_id.search(attributes).group(1)
                    logger.debug(feature_id)
                    #                    if len(groups) != 2:  # should just be ID and Name
                    #                        logger.error('too many groups detected: {}'.format(
                    #                                                                      groups))
                    #                        passed = False
                    #                    (feature_id, feature_name) = groups
                    if not feature_id.startswith(true_id):  # check id
                        logger.error(
                            "gene feature id, should start with "
                            + "{} line {}".format(true_id, lines)
                        )
                        passed = False
        #                    if not feature_name.startswith(true_name):
        #                        logger.error('feature name, should start with ' +
        #                                     '{} line {}'.format(true_name, lines))
        #                        passed = False
        return passed


class protein:
    def __init__(self, detector, **kwargs):
        self.detector = detector
        self.target = detector.target
        self.logger = detector.logger

    def run(self):
        """Run Checks for protein"""
        logger = self.logger
        logger.info("protein check {}".format(self.target))
        if not self.check_protein():
            logger.error("protein file naming checks FAILED")
            return False
        logger.info("ALL protein CHCEKS PASSED")
        return True

    def check_protein(self):
        """accepts a list of annotation attributes split by "."

           https://github.com/LegumeFederation/datastore/issues/23

           checks these file attributes to ensure they are correct
        """
        target = self.target
        logger = self.logger
        attr = os.path.basename(target).split(".")  # split on delimiter
        if len(attr) != 8:  # should be 8 fields
            logger.error("File did not have 7 fields! {}".format(attr))
            return False
        if len(attr[0]) != 5:  # should be 5 letter prefix
            logger.error("File must have 5 letter prefix, not {}".format(attr[0]))
            return False
        if not attr[2].startswith("gnm"):  # should be gnm type
            logger.error("File should have gnm in field 3, not {}".format(attr[2]))
            return False
        gnm_v = attr[2].replace("gnm", "")
        try:
            int(gnm_v)
        except ValueError:  # best way to check for int in python2
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if len(gnm_v.split(".")) > 1:  # check for float
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if not attr[3].startswith("ann"):  # should be gnm type
            logger.error("File should have ann in field 4, not {}".format(attr[2]))
            return False
        ann_v = attr[3].replace("ann", "")
        try:
            int(ann_v)
        except ValueError:  # best way to check for int in python2
            logger.error("ann version must be integer not {}".format(ann_v))
            return False
        if len(ann_v.split(".")) > 1:  # check for float
            logger.error("ann version must be integer not {}".format(gnm_v))
            return False
        if not attr[6] == "faa":  # should be gff3 type
            logger.error("File should be gff3 not {}".format(attr[6]))
            return False
        if not attr[7] == "gz":  # should be gzip compressed
            logger.error("Last field should be gz, not {}".format(attr[6]))
            return False
        return True


class protein_primaryTranscript:
    def __init__(self, detector, **kwargs):
        self.detector = detector
        self.target = detector.target
        self.logger = detector.logger

    def run(self):
        """Run Checks for protein_primaryTranscript"""
        logger = self.logger
        logger.info("protein_primaryTranscript check {}".format(self.target))
        if not self.check_protein_primaryTranscript():
            logger.error("protein_primaryTranscript naming checks FAILED")
            return False
        logger.info("ALL protein_primaryTranscript CHCEKS PASSED")
        return True

    def check_protein_primaryTranscript(self):
        """accepts a list of annotation attributes split by "."

           https://github.com/LegumeFederation/datastore/issues/23

           checks these file attributes to ensure they are correct
        """
        target = self.target
        logger = self.logger
        attr = os.path.basename(target).split(".")  # split on delimiter
        if len(attr) != 8:  # should be 8 fields
            logger.error("File did not have 7 fields! {}".format(attr))
            return False
        if len(attr[0]) != 5:  # should be 5 letter prefix
            logger.error("File must have 5 letter prefix, not {}".format(attr[0]))
            return False
        if not attr[2].startswith("gnm"):  # should be gnm type
            logger.error("File should have gnm in field 3, not {}".format(attr[2]))
            return False
        gnm_v = attr[2].replace("gnm", "")
        try:
            int(gnm_v)
        except ValueError:  # best way to check for int in python2
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if len(gnm_v.split(".")) > 1:  # check for float
            logger.error("gnm version must be integer not {}".format(gnm_v))
            return False
        if not attr[3].startswith("ann"):  # should be gnm type
            logger.error("File should have ann in field 4, not {}".format(attr[2]))
            return False
        ann_v = attr[3].replace("ann", "")
        try:
            int(ann_v)
        except ValueError:  # best way to check for int in python2
            logger.error("ann version must be integer not {}".format(ann_v))
            return False
        if len(ann_v.split(".")) > 1:  # check for float
            logger.error("ann version must be integer not {}".format(gnm_v))
            return False
        if not attr[6] == "faa":  # should be gff3 type
            logger.error("File should be gff3 not {}".format(attr[6]))
            return False
        if not attr[7] == "gz":  # should be gzip compressed
            logger.error("Last field should be gz, not {}".format(attr[6]))
            return False
        return True


class readme_md:  # need to populate this correctly later
    def __init__(self, detector, **kwargs):
        self.detector = detector
        self.target = detector.target
        self.logger = detector.logger
        self.fasta_ids = detector.fasta_ids

    def validate_checksum(self, md5_file, check_me):
        """Get md5 checksum for file and compare to expected"""
        logger = self.logger
        fh = return_filehandle(md5_file)
        hash_md5 = hashlib.md5()
        check_sum_target = ""
        switch = 0
        with fh as copen:
            for line in copen:
                line = line.rstrip()
                if not line or line.startswith("#"):
                    continue
                fields = line.split()
                check_sum = fields[0]
                filename = fields[1]
                logger.debug("check_sum: {}, filename: {}".format(check_sum, filename))
                if not check_sum and filename:
                    logger.error("Could not find sum and name for {}".format(line))
                if filename == os.path.basename(check_me):
                    logger.info("Checksum found for {}".format(filename))
                    check_sum_target = check_sum
                    switch = 1
        if not switch:
            logger.error("Could not find checksum for {}".format(check_me))
            sys.exit(1)
        with open(check_me, "rb") as copen:
            for chunk in iter(lambda: copen.read(4096), b""):  # 4096 buffer
                hash_md5.update(chunk)
        target_sum = hash_md5.hexdigest()  # get sum
        logger.debug(target_sum)
        logger.debug(check_sum_target)
        if target_sum != check_sum_target:  # compare sums
            logger.error(
                (
                    "Checksum for file {} {} ".format(check_me, target_sum)
                    + "did not match {}".format(check_sum_target)
                )
            )
            sys.exit(1)
        logger.info("Checksums checked out, moving on...")

    def validate_doi(self, readme):
        """Parse README.<key>.yml and get publication or dataset DOIs

           Uses http://www.doi.org/factsheets/DOIProxy.html#rest-api
        """
        logger = self.logger
        publication_doi = ""
        dataset_doi = ""
        object_dois = {}
